using UnityEngine;
using System.Collections;

public class GameControllerScript : MonoBehaviour 
{
	private const float NUM_BURST_MOVE = 8;
	
	private GameObject _player;
	private Movement _playerMovement;
	private int _numTotalPickUps;
	private int _numPickUpsLeft;
	
	void Start () 
	{		
		_numTotalPickUps = getTotalGoodPickUps();
		_numPickUpsLeft = _numTotalPickUps;
		
		_player = GameObject.FindGameObjectWithTag("Player");
		_playerMovement = _player.GetComponentInChildren<Movement>();
	}
	
	void Update()
	{
		if (Input.GetButtonDown("Level1"))
		{
			startLevelOne();
		}
		
		if (Input.GetButtonDown("Level2"))
		{
			startLevelTwo();
		}
	}
	
	#region PUBLIC METHODS
	
	public void restartGame()
	{
		iTween.Pause();
		iTween.RotateAdd(_player, iTween.Hash(iT.RotateAdd.y, 420, iT.RotateAdd.time, 1, iT.RotateAdd.easetype, iTween.EaseType.linear, iT.RotateAdd.oncompletetarget, gameObject, iT.RotateAdd.oncomplete, "startLevelOne"));
	}
	
	public void reversePlayer()
	{
		_playerMovement.reverse();	
	}
	
	public void speedBurst()
	{
		_playerMovement.speedBurst(NUM_BURST_MOVE, 2);	
	}
	
	public void superJump()
	{
		_playerMovement.superJump();
	}
	
	public void increasePlayerSpeed(float amount)
	{
		_playerMovement.increaseSpeed(amount);
		_numPickUpsLeft--;
		//print(_numPickUpsLeft);
	}
	#endregion
	
	
	#region PRIVATE METHODS
	
	private void startLevelOne()
	{
		iTween.Stop();
		Application.LoadLevel("main");
	}
	
	private void startLevelTwo()
	{
		iTween.Stop();
		Application.LoadLevel("test");
	}
	
	private int getTotalGoodPickUps()
	{
		GameObject[] allPickUps = GameObject.FindGameObjectsWithTag("GoodPickUp");
		int numPickUps = allPickUps.Length;
		//print(numPickUps);
		return numPickUps;
	}
	#endregion
}
