using UnityEngine;
using System.Collections;

public class Movement : MonoBehaviour 
{
	private const float MIN_MOVE_TIME = .05f;
	
	private int _turnAmount = 0;
	private float _gridSize;
	private float _startYPos;
	private float _tempMoveTime;
	private float _moveTime = .2f;
	private float _turnTime = .28f;
	private float _jumpAmount = 2.5f;
	private bool _jumpFlag;
	private bool _isDownKeyPressed;
	
	private string _currentState;
	
	private GameObject _character;
	
	private void Start () 
	{
		_currentState = PlayerStates.RUNNING;
		
		_character = GameObject.FindGameObjectWithTag("Character");
		_jumpFlag = false;
		_isDownKeyPressed = false;
		_turnAmount = 0;
		_startYPos = _character.transform.position.y;
		_tempMoveTime = _moveTime;
		
		GameObject ground = GameObject.FindGameObjectWithTag("Ground");
		//_gridSize = (ground.transform.localScale.x / ground.renderer.material.mainTextureScale.x) * 10;
		_gridSize = 8;
		//float num = (ground.transform.localScale.x /_gridSize) * 10;
		//ground.renderer.material.SetTextureScale("_MainTex", new Vector2(num, num));
		//ground.renderer.material.mainTextureScale.x = num;
		//print(num);
		_moveTime = (_gridSize / (_gridSize * 4));
		
		iTween.Init(gameObject);
		iTween.Init(_character);
		moveForwardOneGrid();
	}
	
	#region GETTERS AND SETTERS
	
	public int turnAmount 
	{
		get { return _turnAmount; }
		set { _turnAmount = value; }
	}
	
	public bool jumpFlag 
	{
		get { return _jumpFlag; }
		set { _jumpFlag = value; }
	}
	
	public bool isDownKeyPressed 
	{
		get { return _isDownKeyPressed; }
		set { _isDownKeyPressed = value; }
	}
	
	#endregion
	
	#region PUBLIC METHODS
	
	public void reverse()
	{
		speedBurst(-3, -2);
	}
	
	public void increaseSpeed(float amount)
	{
		if (_moveTime > MIN_MOVE_TIME)
			_moveTime -= amount;	
		else
			_moveTime = MIN_MOVE_TIME;
	}
	
	public void superJump()
	{
		float height = 2.5f;
		float time = _moveTime * 2;
		iTween.MoveAdd(_character, iTween.Hash(iT.MoveAdd.y, height, iT.MoveAdd.time, time, iT.MoveAdd.easetype, iTween.EaseType.easeInOutCubic, iT.MoveAdd.oncompletetarget, gameObject, iT.MoveAdd.oncomplete, "superJumpComplete"));
		speedBurst(3, 1.0f);
	}
	
	public void sideStep(int direction)
	{
		if (iTween.Count(gameObject) <= 1 && _currentState == PlayerStates.RUNNING)
		{
			float time = .13f;
			iTween.MoveAdd(gameObject, iTween.Hash(iT.MoveAdd.x, (_gridSize/2) * direction, iT.MoveAdd.time, time, iT.MoveAdd.easetype, iTween.EaseType.easeInOutCubic));
		}
	}
	
	#endregion
	
	#region PRIVATE METHODS
	
	private void superJumpComplete()
	{
		float height = 2.5f;
		float time = _moveTime * 3;
		iTween.MoveAdd(_character, iTween.Hash(iT.MoveAdd.y, height * -1, iT.MoveAdd.time, time, iT.MoveAdd.easetype, iTween.EaseType.easeInOutCubic));
	}
	
	public void speedBurst(float numGridMove, float speedDivider)
	{
		_currentState = PlayerStates.SPEED_BURST;
		iTween.MoveAdd(gameObject, iTween.Hash(iT.MoveBy.z, (_gridSize/2) * numGridMove, iT.MoveBy.time, _moveTime * (numGridMove/speedDivider), iT.MoveBy.easetype, iTween.EaseType.linear, iT.MoveAdd.oncomplete, "speedBurstComplete"));
	}
	
	private void speedBurstComplete()
	{
		_currentState = PlayerStates.RUNNING;
	}
	
	private void duckDown()
	{
		_tempMoveTime = _moveTime;
		_moveTime = .1f;
		float time = .3f;
		float height = -1.5f;
		
		_currentState = PlayerStates.TRANSITIONING;
		iTween.MoveAdd(_character, iTween.Hash(iT.MoveAdd.y, height, iT.MoveAdd.time, time, iT.MoveAdd.easetype, iTween.EaseType.easeInOutCubic, iT.MoveAdd.oncompletetarget, gameObject, iT.MoveAdd.oncomplete, "duckDownComplete"));
	}
	
	private void standBackUp(float height)
	{		
		_moveTime = _tempMoveTime;
		float time = .5f;
		iTween.MoveAdd(_character, iTween.Hash(iT.MoveAdd.y, height * -1, iT.MoveAdd.time, time, iT.MoveAdd.easetype, iTween.EaseType.easeInOutCubic, iT.MoveAdd.oncompletetarget, gameObject, iT.MoveAdd.oncomplete, "resetduckFlag"));
	}
	
	private void jumpUp(float height)
	{
		_currentState = PlayerStates.JUMPING;
		
		iTween.MoveAdd(_character, iTween.Hash(iT.MoveAdd.y, height, iT.MoveAdd.time, height/10, iT.MoveAdd.easetype, iTween.EaseType.easeInOutCubic, iT.MoveAdd.oncompletetarget, gameObject, iT.MoveAdd.oncomplete, "jumpDown", iT.MoveAdd.oncompleteparams, height));
	}
	
	private void jumpDown(float height)
	{
		iTween.MoveAdd(_character, iTween.Hash(iT.MoveTo.y, height * -1, iT.MoveTo.time, height/10, iT.MoveTo.easetype, iTween.EaseType.easeOutCubic, iT.MoveAdd.oncompletetarget, gameObject, iT.MoveAdd.oncomplete, "resetJumpFlag"));
	}
	
	private void makeTurn(float amount)
	{
		_currentState = PlayerStates.TURNING;
		
		iTween.RotateAdd(gameObject, iTween.Hash(iT.RotateAdd.y, amount, iT.RotateAdd.time, _turnTime, iT.RotateAdd.easetype, iTween.EaseType.easeInOutSine, iT.RotateAdd.oncomplete, "onTurnComplete"));
		iTween.Pause("MoveBy");
	}
	
	private void onTurnComplete()
	{
		_currentState = PlayerStates.RUNNING;
		
		_turnAmount = 0;
		iTween.Resume("MoveBy");
	}
	
	private void resetJumpFlag()
	{
		_jumpFlag = false;
		_currentState = PlayerStates.RUNNING;
	}
	
	private void duckDownComplete()
	{
		_currentState = PlayerStates.DUCKING;
	}
	
	private void resetduckFlag()
	{
		_currentState = PlayerStates.RUNNING;
	}
	
	private void moveForwardOneGrid()
	{	
		iTween.MoveBy(gameObject, iTween.Hash(iT.MoveBy.z, _gridSize/2, iT.MoveBy.time, _moveTime, iT.MoveBy.easetype, iTween.EaseType.linear, iT.MoveAdd.oncomplete, "moveForwardOneGrid"));
		
		if (_currentState == PlayerStates.RUNNING)
		{
			if (_turnAmount != 0)
				makeTurn(_turnAmount);
			
			if (_jumpFlag)
				jumpUp(_jumpAmount);
			
			if (_isDownKeyPressed)
				duckDown();
		}
		if (_currentState == PlayerStates.DUCKING && !_isDownKeyPressed)
		{
				standBackUp(-1.5f);	
		}
	}
	
	#endregion
}
